#include "polyhedralcone.h"

#include "lp.h"
#include "subspace.h"

//--------------------------------
// PolyhedralCone
//--------------------------------


void PolyhedralCone::reduce()
{
  //..... linearitySpace could grow..
  removeRedundantRows(&halfSpaces,&linearitySpace);
}


PolyhedralCone::PolyhedralCone(int ambientDimension):
  n(ambientDimension)
{
}


PolyhedralCone::PolyhedralCone(IntegerVectorList const &halfSpaces_, IntegerVectorList const &linearitySpace_, int ambientDimension):
  halfSpaces(halfSpaces_),
  linearitySpace(linearitySpace_)
{
  n=ambientDimension;
  if(n==-1)
    {
      if(!halfSpaces_.empty())
	n=halfSpaces_.begin()->size();
      else if(!linearitySpace_.empty())
	n=linearitySpace_.begin()->size();
      else
	{
	  assert(0);
	}
    }
  reduce();
}


IntegerVector PolyhedralCone::relativeInteriorPoint(bool strictlyPositive)const
{
  IntegerVector ret;
  IntegerVectorList g=linearitySpace;
  int numberOfEqualities=g.size();
  g.insert(g.end(),halfSpaces.begin(),halfSpaces.end());
  IntegerVector equalitySet(g.size());
  for(int i=0;i<numberOfEqualities;i++)equalitySet[i]=1;
  bool r=interiorPoint(g,ret,strictlyPositive,&equalitySet);
  assert(r);// It is alway possible to find a relative interior point in a cone

  for(IntegerVectorList::const_iterator i=linearitySpace.begin();i!=linearitySpace.end();i++)
    {
      assert(dotLong(*i,ret)==0);
    }

  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    {
      if(!(dotLong(*i,ret)>0))
	{
	  fprintf(stderr,"PolyhedralCone::relativeInteriorPoint() : halfSpaces not reduced or mistake in cdd interface!!!\n");
	}
    }

  return ret;
}


int PolyhedralCone::ambientDimension()const
{
  return n;
}


int PolyhedralCone::dimension()const
{
  //      return ambientDimension()-linearitySpace.size();
  return ambientDimension()-Subspace(linearitySpace).dimension();
}


bool PolyhedralCone::isZero()const
{
  return dimension()==0;
}


PolyhedralCone intersection(const PolyhedralCone &a, const PolyhedralCone &b)
{
  assert(a.ambientDimension()==b.ambientDimension());
  IntegerVectorList halfSpaces=a.halfSpaces;
  halfSpaces.insert(halfSpaces.end(),b.halfSpaces.begin(),b.halfSpaces.end());
  IntegerVectorList linearitySpace=a.linearitySpace;
  linearitySpace.insert(linearitySpace.end(),b.linearitySpace.begin(),b.linearitySpace.end());
  return PolyhedralCone(halfSpaces,linearitySpace,a.ambientDimension());
}


PolyhedralCone PolyhedralCone::positiveOrthant(int dimension)
{
  IntegerVectorList halfSpaces;

  for(int i=0;i<dimension;i++)halfSpaces.push_back(IntegerVector::standardVector(dimension,i));

  return PolyhedralCone(halfSpaces,IntegerVectorList(),dimension);
}
