#include "reversesearch.h"

#include "buchberger.h"
#include "wallideal.h"
#include "printer.h"
#include "lp.h"

bool ReverseSearch::computeSearchEdge(PolynomialSet &groebnerBasis, IntegerVector *edge)
{

  IntegerVectorList normals=wallInequalities(groebnerBasis);
  normals.sort(LexicographicTermOrder());// Is this needed to make the interior point computation deterministic?

  //  fprintf(Stderr,"\nBasis with no interior point:\n");
  //  AsciiPrinter(Stderr).printPolynomialSet(groebnerBasis);

  if(normals.empty())
    {
      fprintf(Stderr,"WARNING: reversesearch.cpp - No normals. Maybe your ideal is a monomial ideal.\n");
    }

  IntegerVectorList::const_iterator r=shootRay(normals);

  if(r!=normals.end())
    {
      *edge=*r;
      return true;
    }
  return false;

  /*  
  IntegerVectorList normals=wallNormals(groebnerBasis);
  normals.sort(LexicographicTermOrder());

  for(IntegerVectorList::const_iterator i=normals.begin();i!=normals.end();i++)
    if(termOrder(*i,*i-*i))
      if(isFacet(normals,i))
	if(wallContainsPositiveVector(*i))
	  {
            *edge=*i;
            return true;
          }
  return false;
  */
}


int ReverseSearch::treeSize(const PolynomialSet &groebnerBasis)
{
  {
    static int n;
    n++;
    if(!(n%10))
      fprintf(Stderr,"%i\n",n);
  }

  int s=1;
  if(!targetBasis(groebnerBasis)){breaked=true;return s;}

  IntegerVectorList flipable=wallFlipableNormals(groebnerBasis);

  for(IntegerVectorList::iterator i=flipable.begin();i!=flipable.end();i++)
    {
      if(!termOrder(*i,*i-*i))
	{
	  PolynomialSet neighbour=flip(groebnerBasis,*i);
	  
	  IntegerVector edge;
	  if(computeSearchEdge(neighbour,&edge))
	    if(dependent(edge,*i))
	      {
		s+=treeSize(neighbour);
		if(breaked)return s;
	      }
	}
    }

  return s;
}


PolynomialSet ReverseSearch::findRoot(PolynomialSet groebnerBasis)
{
  //  fprintf(Stderr,"Computing root\n");
  //  buchberger(&groebnerBasis,termOrder);

  IntegerVector edge;
  while(computeSearchEdge(groebnerBasis,&edge))
    {
      AsciiPrinter(Stderr).printVector(edge);
      groebnerBasis=flip(groebnerBasis,edge);
    }

  //  fprintf(Stderr,"Done computing root\n");
  return groebnerBasis;
}


ReverseSearch::ReverseSearch(const TermOrder &termOrder_):
  numberOfVertices(0),
  numberOfEdges(0),
  termOrder(termOrder_)
{
}


void ReverseSearch::enumerate(const PolynomialSet &groebnerBasis)
{
  breaked=false;

  PolynomialSet root=findRoot(groebnerBasis);


  targetBeginEnumeration(groebnerBasis);
  numberOfVertices=treeSize(root);
  targetEndEnumeration();

  fprintf(Stderr,"numberOfVertices:%i\n",numberOfVertices);
}
